import 'dart:async';

typedef EventListener<Event> = FutureOr<void> Function(Event event);

/// An object that provides access to a stream of states over time.
abstract class Streamable<State> {
  /// The current [stream] of states.
  Stream<State> get stream;
}

/// A [Streamable] that provides synchronous access to the current [state].
abstract class StateStreamable<State> implements Streamable<State> {
  /// The current [state].
  State get state;
}

abstract class Emittable<Event> {
  void emit(Event event);
}

abstract class SelectiveListenable {
  StreamSubscription<T> on<T>(EventListener<T> listener);
}

/// An object that must be closed when no longer in use.
abstract class Closable {
  /// Closes the current instance.
  /// The returned future completes when the instance has been closed.
  FutureOr<void> close();

  /// Whether the object is closed.
  ///
  /// An object is considered closed once [close] is called.
  bool get isClosed;
}

/// A [StateStreamable] that must be closed when no longer in use.
abstract class StateStreamableSource<State> implements StateStreamable<State>, Closable {}

abstract class EventStreamableSource<Event> implements Streamable<Event>, SelectiveListenable, Closable {}

abstract class StoreBase<State> implements StateStreamableSource<State>, Emittable<State> {}

abstract class EventBusBase<Event> implements EventStreamableSource<Event>, Emittable<Event> {}
